//
//  SpaceshipScene.m
//  Space Simulator
//
//  Created by Simon Rummert on 14.08.13.
//  Copyright (c) 2013 Simon Rummert. All rights reserved.
//

#import "SpaceshipScene.h"

@interface SpaceshipScene ()
@property BOOL contentCreated;

@end

@implementation SpaceshipScene

static const uint32_t shipCategory        =  0x1 << 1;
static const uint32_t starCategory        =  0x1 << 2;

- (void)didMoveToView:(SKView *)view
{
    if (!self.contentCreated)
    {
        [self createSceneContents];
        self.contentCreated = YES;
    }
}

- (void)createSceneContents
{
    self.backgroundColor = [SKColor blackColor];
    self.scaleMode = SKSceneScaleModeAspectFit;
    SKSpriteNode *spaceship = [self newSpaceship];
    spaceship.position = CGPointMake(CGRectGetMidX(self.frame),                              CGRectGetMidY(self.frame)-150);
    [self addChild:spaceship];
    SKAction *makeStars = [SKAction sequence: @[
                                                [SKAction performSelector:@selector(addStar) onTarget:self],
                                                [SKAction waitForDuration:0.10 withRange:0.15]
                                                ]];
    [self runAction: [SKAction repeatActionForever:makeStars]];
    self.physicsWorld.contactDelegate = self;
}

- (SKSpriteNode *)newSpaceship
{ 
    SKSpriteNode *spaceship = [SKSpriteNode spriteNodeWithImageNamed:@"Spaceship.png"];
    
    spaceship.xScale = 0.25;
    spaceship.yScale = 0.25;
    
    SKAction *hover = [SKAction sequence:@[
                                           [SKAction waitForDuration:1.0],
                                           [SKAction moveByX:100 y:50.0 duration:1.0],
                                           [SKAction waitForDuration:1.0],
                                           [SKAction moveByX:-100.0 y:-50 duration:1.0]]];
    [spaceship runAction: [SKAction repeatActionForever:hover]];
    
    
    SKEmitterNode *rockettrail = [self newFireEmitter];
    
    //the y-position needs to be slightly behind the spaceship
    rockettrail.position = CGPointMake(0, -180.0);
    
    //scaling the particlesystem
    rockettrail.xScale = 2.0;
    rockettrail.yScale = 2.0;
    
    //changing the targetnode from spaceship to scene so that it gets influenced by movement
    rockettrail.targetNode = self.scene;
    
    [spaceship addChild:rockettrail];
    
    
    spaceship.physicsBody = [SKPhysicsBody bodyWithCircleOfRadius:0.4*spaceship.size.width];
    spaceship.physicsBody.dynamic = NO;
    spaceship.physicsBody.categoryBitMask = shipCategory;
    spaceship.physicsBody.collisionBitMask = shipCategory | starCategory;
    spaceship.physicsBody.contactTestBitMask = shipCategory | starCategory;
    
    return spaceship;
}



static inline CGFloat skRandf() {
    return rand() / (CGFloat) RAND_MAX;
}

static inline CGFloat skRand(CGFloat low, CGFloat high) {
    return skRandf() * (high - low) + low;
}

- (void)addStar
{
    SKSpriteNode *star = [[SKSpriteNode alloc] initWithImageNamed:@"Star.png"];
    star.xScale = 0.25;
    star.yScale = 0.25;
    star.position = CGPointMake(skRand(0, self.size.width), self.size.height);
    star.name = @"star";
    star.physicsBody = [SKPhysicsBody bodyWithCircleOfRadius:0.5*star.size.width];
    
    /* If you want rebounding stars instead of explosions.       *
     * You need to remove collision detection in order to do so  *
     *                                                           *
     //star.physicsBody.usesPreciseCollisionDetection = YES;     */
    
    
    [self addChild:star];
}

-(void)didSimulatePhysics
{
    [self enumerateChildNodesWithName:@"star" usingBlock:^(SKNode *node, BOOL *stop) {
        if (node.position.y < 0)
            [node removeFromParent];
        
    }];
}

//-----------------------EXERCISE 1----------------------------------
-(void)didBeginContact:(SKPhysicsContact *)contact
{
    //insantiate new explosion on contact
    SKEmitterNode *ex = [self newExplosion];
    
    //set the position of contact
    ex.position = contact.bodyB.node.position;
    
    [contact.bodyB.node removeFromParent];
    
}

-(SKEmitterNode *)newExplosion
{
    
    //instantiate explosion emitter
    SKEmitterNode *explosion = [[SKEmitterNode alloc] init];
    
    //TODO: set the right properties for your particle system!
    [explosion setParticleTexture:[SKTexture textureWithImageNamed:@"spark.png"]];
    [explosion setParticleColor:[UIColor brownColor]];
    [explosion setNumParticlesToEmit:100];
    [explosion setParticleBirthRate:450];
    [explosion setParticleLifetime:2];
    [explosion setEmissionAngleRange:360];
    [explosion setParticleSpeed:100];
    [explosion setParticleSpeedRange:50];
    [explosion setXAcceleration:0];
    [explosion setYAcceleration:0];
    [explosion setParticleAlpha:0.8];
    [explosion setParticleAlphaRange:0.2];
    [explosion setParticleAlphaSpeed:-0.5];
    [explosion setParticleScale:0.75];
    [explosion setParticleScaleRange:0.4];
    [explosion setParticleScaleSpeed:-0.5];
    [explosion setParticleRotation:0];
    [explosion setParticleRotationRange:0];
    [explosion setParticleRotationSpeed:0];
    
    [explosion setParticleColorBlendFactor:1];
    [explosion setParticleColorBlendFactorRange:0];
    [explosion setParticleColorBlendFactorSpeed:0];
    [explosion setParticleBlendMode:SKBlendModeAdd];
   
    //add this node to parent node
    [self addChild:explosion];
    
    return explosion;
}
//-----------------------EXERCISE 1 END------------------------------


//-----------------------EXERCISE 2----------------------------------
- (SKEmitterNode *) newFireEmitter
{
    NSString *firePath = [[NSBundle mainBundle] pathForResource:@"Fire" ofType:@"sks"];
    SKEmitterNode *fire = [NSKeyedUnarchiver unarchiveObjectWithFile:firePath];
    return fire;
}
//-----------------------EXERCISE 2 END------------------------------

@end